# -*- coding: utf-8 -*-
# pylint: disable=invalid-name,line-too-long,protected-access,too-many-locals
# pylint: disable=too-many-lines
"""
Primitive ``ctypes`` prototypes for the NAG Library `correg` Chapter.
"""

# NAG Copyright 2017-2019.

from ..base import utils

corrmat_nearest = utils._EngineState._get_symbol('g02aaf')
corrmat_nearest.restype = None
corrmat_nearest.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, g[ldg, n].
    utils._BYREF_INT, # Input, ldg.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, errtol.
    utils._BYREF_INT, # Input, maxits.
    utils._BYREF_INT, # Input, maxit.
    utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, n].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Output, iter.
    utils._BYREF_INT, # Output, feval.
    utils._BYREF_FLOAT64, # Output, nrmgrd.
    utils._BYREF_INT, # Input/Output, ifail.
)

corrmat_nearest_bounded = utils._EngineState._get_symbol('g02abf')
corrmat_nearest_bounded.restype = None
corrmat_nearest_bounded.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, g[ldg, n].
    utils._BYREF_INT, # Input, ldg.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_STRING, # Input, opt(len=1).
    utils._BYREF_FLOAT64, # Input, alpha.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, w[n].
    utils._BYREF_FLOAT64, # Input, errtol.
    utils._BYREF_INT, # Input, maxits.
    utils._BYREF_INT, # Input, maxit.
    utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, n].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Output, iter.
    utils._BYREF_INT, # Output, feval.
    utils._BYREF_FLOAT64, # Output, nrmgrd.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of opt.
)

corrmat_nearest_kfactor = utils._EngineState._get_symbol('g02aef')
corrmat_nearest_kfactor.restype = None
corrmat_nearest_kfactor.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, g[ldg, *].
    utils._BYREF_INT, # Input, ldg.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, k.
    utils._BYREF_FLOAT64, # Input, errtol.
    utils._BYREF_INT, # Input, maxit.
    utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, *].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Output, iter.
    utils._BYREF_INT, # Output, feval.
    utils._BYREF_FLOAT64, # Output, nrmpgd.
    utils._BYREF_INT, # Input/Output, ifail.
)

corrmat_h_weight = utils._EngineState._get_symbol('g02ajf')
corrmat_h_weight.restype = None
corrmat_h_weight.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, g[ldg, *].
    utils._BYREF_INT, # Input, ldg.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, alpha.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, h[ldh, *].
    utils._BYREF_INT, # Input, ldh.
    utils._BYREF_FLOAT64, # Input, errtol.
    utils._BYREF_INT, # Input, maxit.
    utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, *].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Output, iter.
    utils._BYREF_FLOAT64, # Output, norm.
    utils._BYREF_INT, # Input/Output, ifail.
)

corrmat_nearest_rank = utils._EngineState._get_symbol('g02akf')
corrmat_nearest_rank.restype = None
corrmat_nearest_rank.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, g[ldg, n].
    utils._BYREF_INT, # Input, ldg.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, rank.
    utils._BYREF_FLOAT64, # Input, errtol.
    utils._BYREF_FLOAT64, # Input, ranktol.
    utils._BYREF_INT, # Input, maxits.
    utils._BYREF_INT, # Input, maxit.
    utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, n].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_FLOAT64, # Output, f.
    utils._BYREF_FLOAT64, # Output, rankerr.
    utils._BYREF_INT, # Output, nsub.
    utils._BYREF_INT, # Input/Output, ifail.
)

corrmat_shrinking = utils._EngineState._get_symbol('g02anf')
corrmat_shrinking.restype = None
corrmat_shrinking.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, g[ldg, n].
    utils._BYREF_INT, # Input, ldg.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, k.
    utils._BYREF_FLOAT64, # Input, errtol.
    utils._BYREF_FLOAT64, # Input, eigtol.
    utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, n].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_FLOAT64, # Output, alpha.
    utils._BYREF_INT, # Output, iter.
    utils._BYREF_FLOAT64, # Output, eigmin.
    utils._BYREF_FLOAT64, # Output, norm.
    utils._BYREF_INT, # Input/Output, ifail.
)

corrmat_target = utils._EngineState._get_symbol('g02apf')
corrmat_target.restype = None
corrmat_target.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, g[ldg, n].
    utils._BYREF_INT, # Input, ldg.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, theta.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, h[ldh, n].
    utils._BYREF_INT, # Input, ldh.
    utils._BYREF_FLOAT64, # Input, errtol.
    utils._BYREF_FLOAT64, # Input, eigtol.
    utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, n].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_FLOAT64, # Output, alpha.
    utils._BYREF_INT, # Output, iter.
    utils._BYREF_FLOAT64, # Output, eigmin.
    utils._BYREF_FLOAT64, # Output, norm.
    utils._BYREF_INT, # Input/Output, ifail.
)

corrmat_fixed = utils._EngineState._get_symbol('g02asf')
corrmat_fixed.restype = None
corrmat_fixed.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input/Output, g[ldg, n].
    utils._BYREF_INT, # Input, ldg.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, alpha.
    utils._EngineIntArrayType.c_type, # Input, h[ldh, n].
    utils._BYREF_INT, # Input, ldh.
    utils._BYREF_FLOAT64, # Input, errtol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Output, x[ldx, n].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Output, its.
    utils._BYREF_FLOAT64, # Output, fnorm.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_pearson = utils._EngineState._get_symbol('g02baf')
coeffs_pearson.restype = None
coeffs_pearson.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[m].
    utils._EngineFloat64ArrayType.c_type, # Output, std[m].
    utils._EngineFloat64ArrayType.c_type, # Output, ssp[ldssp, m].
    utils._BYREF_INT, # Input, ldssp.
    utils._EngineFloat64ArrayType.c_type, # Output, r[ldr, m].
    utils._BYREF_INT, # Input, ldr.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_pearson_miss_case = utils._EngineState._get_symbol('g02bbf')
coeffs_pearson_miss_case.restype = None
coeffs_pearson_miss_case.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input/Output, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xmiss[m].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[m].
    utils._EngineFloat64ArrayType.c_type, # Output, std[m].
    utils._EngineFloat64ArrayType.c_type, # Output, ssp[ldssp, m].
    utils._BYREF_INT, # Input, ldssp.
    utils._EngineFloat64ArrayType.c_type, # Output, r[ldr, m].
    utils._BYREF_INT, # Input, ldr.
    utils._BYREF_INT, # Output, ncases.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_pearson_miss_pair = utils._EngineState._get_symbol('g02bcf')
coeffs_pearson_miss_pair.restype = None
coeffs_pearson_miss_pair.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input, xmiss[m].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[m].
    utils._EngineFloat64ArrayType.c_type, # Output, std[m].
    utils._EngineFloat64ArrayType.c_type, # Output, ssp[ldssp, m].
    utils._BYREF_INT, # Input, ldssp.
    utils._EngineFloat64ArrayType.c_type, # Output, r[ldr, m].
    utils._BYREF_INT, # Input, ldr.
    utils._BYREF_INT, # Output, ncases.
    utils._EngineFloat64ArrayType.c_type, # Output, cnt[ldcnt, m].
    utils._BYREF_INT, # Input, ldcnt.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_zero = utils._EngineState._get_symbol('g02bdf')
coeffs_zero.restype = None
coeffs_zero.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[m].
    utils._EngineFloat64ArrayType.c_type, # Output, std[m].
    utils._EngineFloat64ArrayType.c_type, # Output, sspz[ldsspz, m].
    utils._BYREF_INT, # Input, ldsspz.
    utils._EngineFloat64ArrayType.c_type, # Output, rz[ldrz, m].
    utils._BYREF_INT, # Input, ldrz.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_zero_miss_case = utils._EngineState._get_symbol('g02bef')
coeffs_zero_miss_case.restype = None
coeffs_zero_miss_case.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input/Output, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xmiss[m].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[m].
    utils._EngineFloat64ArrayType.c_type, # Output, std[m].
    utils._EngineFloat64ArrayType.c_type, # Output, sspz[ldsspz, m].
    utils._BYREF_INT, # Input, ldsspz.
    utils._EngineFloat64ArrayType.c_type, # Output, rz[ldrz, m].
    utils._BYREF_INT, # Input, ldrz.
    utils._BYREF_INT, # Output, ncases.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_zero_miss_pair = utils._EngineState._get_symbol('g02bff')
coeffs_zero_miss_pair.restype = None
coeffs_zero_miss_pair.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input, xmiss[m].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[m].
    utils._EngineFloat64ArrayType.c_type, # Output, std[m].
    utils._EngineFloat64ArrayType.c_type, # Output, sspz[ldsspz, m].
    utils._BYREF_INT, # Input, ldsspz.
    utils._EngineFloat64ArrayType.c_type, # Output, rz[ldrz, m].
    utils._BYREF_INT, # Input, ldrz.
    utils._BYREF_INT, # Output, ncases.
    utils._EngineFloat64ArrayType.c_type, # Output, cnt[ldcnt, m].
    utils._BYREF_INT, # Input, ldcnt.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_pearson_subset = utils._EngineState._get_symbol('g02bgf')
coeffs_pearson_subset.restype = None
coeffs_pearson_subset.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, nvars.
    utils._EngineIntArrayType.c_type, # Input, kvar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, std[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, ssp[ldssp, nvars].
    utils._BYREF_INT, # Input, ldssp.
    utils._EngineFloat64ArrayType.c_type, # Output, r[ldr, nvars].
    utils._BYREF_INT, # Input, ldr.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_pearson_subset_miss_case = utils._EngineState._get_symbol('g02bhf')
coeffs_pearson_subset_miss_case.restype = None
coeffs_pearson_subset_miss_case.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input/Output, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xmiss[m].
    utils._BYREF_INT, # Input, mistyp.
    utils._BYREF_INT, # Input, nvars.
    utils._EngineIntArrayType.c_type, # Input, kvar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, std[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, ssp[ldssp, nvars].
    utils._BYREF_INT, # Input, ldssp.
    utils._EngineFloat64ArrayType.c_type, # Output, r[ldr, nvars].
    utils._BYREF_INT, # Input, ldr.
    utils._BYREF_INT, # Output, ncases.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_pearson_subset_miss_pair = utils._EngineState._get_symbol('g02bjf')
coeffs_pearson_subset_miss_pair.restype = None
coeffs_pearson_subset_miss_pair.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input, xmiss[m].
    utils._BYREF_INT, # Input, nvars.
    utils._EngineIntArrayType.c_type, # Input, kvar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, std[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, ssp[ldssp, nvars].
    utils._BYREF_INT, # Input, ldssp.
    utils._EngineFloat64ArrayType.c_type, # Output, r[ldr, nvars].
    utils._BYREF_INT, # Input, ldr.
    utils._BYREF_INT, # Output, ncases.
    utils._EngineFloat64ArrayType.c_type, # Output, cnt[ldcnt, nvars].
    utils._BYREF_INT, # Input, ldcnt.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_zero_subset = utils._EngineState._get_symbol('g02bkf')
coeffs_zero_subset.restype = None
coeffs_zero_subset.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, nvars.
    utils._EngineIntArrayType.c_type, # Input, kvar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, std[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, sspz[ldsspz, nvars].
    utils._BYREF_INT, # Input, ldsspz.
    utils._EngineFloat64ArrayType.c_type, # Output, rz[ldrz, nvars].
    utils._BYREF_INT, # Input, ldrz.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_zero_subset_miss_case = utils._EngineState._get_symbol('g02blf')
coeffs_zero_subset_miss_case.restype = None
coeffs_zero_subset_miss_case.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input/Output, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xmiss[m].
    utils._BYREF_INT, # Input, mistyp.
    utils._BYREF_INT, # Input, nvars.
    utils._EngineIntArrayType.c_type, # Input, kvar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, std[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, sspz[ldsspz, nvars].
    utils._BYREF_INT, # Input, ldsspz.
    utils._EngineFloat64ArrayType.c_type, # Output, rz[ldrz, nvars].
    utils._BYREF_INT, # Input, ldrz.
    utils._BYREF_INT, # Output, ncases.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_zero_subset_miss_pair = utils._EngineState._get_symbol('g02bmf')
coeffs_zero_subset_miss_pair.restype = None
coeffs_zero_subset_miss_pair.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input, xmiss[m].
    utils._BYREF_INT, # Input, nvars.
    utils._EngineIntArrayType.c_type, # Input, kvar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, std[nvars].
    utils._EngineFloat64ArrayType.c_type, # Output, sspz[ldsspz, nvars].
    utils._BYREF_INT, # Input, ldsspz.
    utils._EngineFloat64ArrayType.c_type, # Output, rz[ldrz, nvars].
    utils._BYREF_INT, # Input, ldrz.
    utils._BYREF_INT, # Output, ncases.
    utils._EngineFloat64ArrayType.c_type, # Output, cnt[ldcnt, nvars].
    utils._BYREF_INT, # Input, ldcnt.
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_kspearman_overwrite = utils._EngineState._get_symbol('g02bnf')
coeffs_kspearman_overwrite.restype = None
coeffs_kspearman_overwrite.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, itype.
    utils._EngineFloat64ArrayType.c_type, # Output, rr[ldrr, m].
    utils._BYREF_INT, # Input, ldrr.
    utils._EngineIntArrayType.c_type, # Workspace, kworka[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworkb[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work1[m].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work2[m].
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_kspearman_miss_case_overwrite = utils._EngineState._get_symbol('g02bpf')
coeffs_kspearman_miss_case_overwrite.restype = None
coeffs_kspearman_miss_case_overwrite.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input/Output, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xmiss[m].
    utils._BYREF_INT, # Input, itype.
    utils._EngineFloat64ArrayType.c_type, # Output, rr[ldrr, m].
    utils._BYREF_INT, # Input, ldrr.
    utils._BYREF_INT, # Output, ncases.
    utils._EngineIntArrayType.c_type, # Output, incase[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworka[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworkb[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworkc[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work1[m].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work2[m].
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_kspearman = utils._EngineState._get_symbol('g02bqf')
coeffs_kspearman.restype = None
coeffs_kspearman.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, itype.
    utils._EngineFloat64ArrayType.c_type, # Output, rr[ldrr, m].
    utils._BYREF_INT, # Input, ldrr.
    utils._EngineIntArrayType.c_type, # Workspace, kworka[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworkb[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work1[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work2[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_kspearman_miss_case = utils._EngineState._get_symbol('g02brf')
coeffs_kspearman_miss_case.restype = None
coeffs_kspearman_miss_case.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input/Output, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xmiss[m].
    utils._BYREF_INT, # Input, itype.
    utils._EngineFloat64ArrayType.c_type, # Output, rr[ldrr, m].
    utils._BYREF_INT, # Input, ldrr.
    utils._BYREF_INT, # Output, ncases.
    utils._EngineIntArrayType.c_type, # Output, incase[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworka[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworkb[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworkc[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work1[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work2[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

coeffs_kspearman_miss_pair = utils._EngineState._get_symbol('g02bsf')
coeffs_kspearman_miss_pair.restype = None
coeffs_kspearman_miss_pair.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input, miss[m].
    utils._EngineFloat64ArrayType.c_type, # Input, xmiss[m].
    utils._BYREF_INT, # Input, itype.
    utils._EngineFloat64ArrayType.c_type, # Output, rr[ldrr, m].
    utils._BYREF_INT, # Input, ldrr.
    utils._BYREF_INT, # Output, ncases.
    utils._EngineFloat64ArrayType.c_type, # Output, cnt[ldcnt, m].
    utils._BYREF_INT, # Input, ldcnt.
    utils._EngineIntArrayType.c_type, # Workspace, kworka[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworkb[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworkc[n].
    utils._EngineIntArrayType.c_type, # Workspace, kworkd[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work1[n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, work2[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

ssqmat_update = utils._EngineState._get_symbol('g02btf')
ssqmat_update.restype = None
ssqmat_update.argtypes = (
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_FLOAT64, # Input, wt.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m*incx].
    utils._BYREF_INT, # Input, incx.
    utils._BYREF_FLOAT64, # Input/Output, sw.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xbar[m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, c[(m*m+m)/2].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
)

ssqmat = utils._EngineState._get_symbol('g02buf')
ssqmat.restype = None
ssqmat.argtypes = (
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_FLOAT64, # Output, sw.
    utils._EngineFloat64ArrayType.c_type, # Output, wmean[m].
    utils._EngineFloat64ArrayType.c_type, # Output, c[(m*m+m)/2].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

ssqmat_to_corrmat = utils._EngineState._get_symbol('g02bwf')
ssqmat_to_corrmat.restype = None
ssqmat_to_corrmat.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, r[(m*m+m)/2].
    utils._BYREF_INT, # Input/Output, ifail.
)

corrmat = utils._EngineState._get_symbol('g02bxf')
corrmat.restype = None
corrmat.argtypes = (
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[m].
    utils._EngineFloat64ArrayType.c_type, # Output, std[m].
    utils._EngineFloat64ArrayType.c_type, # Output, v[ldv, m].
    utils._BYREF_INT, # Input, ldv.
    utils._EngineFloat64ArrayType.c_type, # Output, r[ldv, m].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

corrmat_partial = utils._EngineState._get_symbol('g02byf')
corrmat_partial.restype = None
corrmat_partial.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, ny.
    utils._BYREF_INT, # Input, nx.
    utils._EngineIntArrayType.c_type, # Input, isz[m].
    utils._EngineFloat64ArrayType.c_type, # Input, r[ldr, m].
    utils._BYREF_INT, # Input, ldr.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, ny].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[ny*nx+nx*(nx+1)/2].
    utils._BYREF_INT, # Input/Output, ifail.
)

ssqmat_combine = utils._EngineState._get_symbol('g02bzf')
ssqmat_combine.restype = None
ssqmat_combine.argtypes = (
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_INT, # Input, m.
    utils._BYREF_FLOAT64, # Input/Output, xsw.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xmean[m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xc[*].
    utils._BYREF_FLOAT64, # Input, ysw.
    utils._EngineFloat64ArrayType.c_type, # Input, ymean[m].
    utils._EngineFloat64ArrayType.c_type, # Input, yc[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
)

linregs_const = utils._EngineState._get_symbol('g02caf')
linregs_const.restype = None
linregs_const.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Output, result[20].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregs_noconst = utils._EngineState._get_symbol('g02cbf')
linregs_noconst.restype = None
linregs_noconst.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Output, result[20].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregs_const_miss = utils._EngineState._get_symbol('g02ccf')
linregs_const_miss.restype = None
linregs_const_miss.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_FLOAT64, # Input, xmiss.
    utils._BYREF_FLOAT64, # Input, ymiss.
    utils._EngineFloat64ArrayType.c_type, # Output, result[21].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregs_noconst_miss = utils._EngineState._get_symbol('g02cdf')
linregs_noconst_miss.restype = None
linregs_noconst_miss.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_FLOAT64, # Input, xmiss.
    utils._BYREF_FLOAT64, # Input, ymiss.
    utils._EngineFloat64ArrayType.c_type, # Output, result[21].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_service_select = utils._EngineState._get_symbol('g02cef')
linregm_service_select.restype = None
linregm_service_select.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, xbar[n].
    utils._EngineFloat64ArrayType.c_type, # Input, std[n].
    utils._EngineFloat64ArrayType.c_type, # Input, ssp[ldssp, n].
    utils._BYREF_INT, # Input, ldssp.
    utils._EngineFloat64ArrayType.c_type, # Input, r[ldr, n].
    utils._BYREF_INT, # Input, ldr.
    utils._BYREF_INT, # Input, m.
    utils._EngineIntArrayType.c_type, # Input, korder[m].
    utils._EngineFloat64ArrayType.c_type, # Output, xbar2[m].
    utils._EngineFloat64ArrayType.c_type, # Output, std2[m].
    utils._EngineFloat64ArrayType.c_type, # Output, ssp2[ldssp2, m].
    utils._BYREF_INT, # Input, ldssp2.
    utils._EngineFloat64ArrayType.c_type, # Output, r2[ldr2, m].
    utils._BYREF_INT, # Input, ldr2.
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_service_reorder = utils._EngineState._get_symbol('g02cff')
linregm_service_reorder.restype = None
linregm_service_reorder.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineIntArrayType.c_type, # Input, korder[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xbar[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, std[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ssp[ldssp, n].
    utils._BYREF_INT, # Input, ldssp.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, r[ldr, n].
    utils._BYREF_INT, # Input, ldr.
    utils._EngineIntArrayType.c_type, # Workspace, kwork[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_coeffs_const = utils._EngineState._get_symbol('g02cgf')
linregm_coeffs_const.restype = None
linregm_coeffs_const.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, k1.
    utils._BYREF_INT, # Input, k.
    utils._EngineFloat64ArrayType.c_type, # Input, xbar[k+1].
    utils._EngineFloat64ArrayType.c_type, # Input, ssp[ldssp, k+1].
    utils._BYREF_INT, # Input, ldssp.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, r[ldr, k+1].
    utils._BYREF_INT, # Input, ldr.
    utils._EngineFloat64ArrayType.c_type, # Output, result[13].
    utils._EngineFloat64ArrayType.c_type, # Output, coef[ldcoef, 3].
    utils._BYREF_INT, # Input, ldcoef.
    utils._EngineFloat64ArrayType.c_type, # Output, con[3].
    utils._EngineFloat64ArrayType.c_type, # Output, rinv[ldrinv, k].
    utils._BYREF_INT, # Input, ldrinv.
    utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, k].
    utils._BYREF_INT, # Input, ldc.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wkz[ldwkz, k].
    utils._BYREF_INT, # Input, ldwkz.
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_coeffs_noconst = utils._EngineState._get_symbol('g02chf')
linregm_coeffs_noconst.restype = None
linregm_coeffs_noconst.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, k1.
    utils._BYREF_INT, # Input, k.
    utils._EngineFloat64ArrayType.c_type, # Input, sspz[ldsspz, k+1].
    utils._BYREF_INT, # Input, ldsspz.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rz[ldrz, k+1].
    utils._BYREF_INT, # Input, ldrz.
    utils._EngineFloat64ArrayType.c_type, # Output, result[13].
    utils._EngineFloat64ArrayType.c_type, # Output, coef[ldcoef, 3].
    utils._BYREF_INT, # Input, ldcoef.
    utils._EngineFloat64ArrayType.c_type, # Output, rznv[ldrznv, k].
    utils._BYREF_INT, # Input, ldrznv.
    utils._EngineFloat64ArrayType.c_type, # Output, cz[ldcz, k].
    utils._BYREF_INT, # Input, ldcz.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wkz[ldwkz, k].
    utils._BYREF_INT, # Input, ldwkz.
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_fit = utils._EngineState._get_symbol('g02daf')
linregm_fit.restype = None
linregm_fit.argtypes = (
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, m.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_FLOAT64, # Output, rss.
    utils._BYREF_INT, # Output, idf.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Output, res[n].
    utils._EngineFloat64ArrayType.c_type, # Output, h[n].
    utils._EngineFloat64ArrayType.c_type, # Output, q[ldq, ip+1].
    utils._BYREF_INT, # Input, ldq.
    utils._BYREF_LOGICAL, # Output, svd.
    utils._BYREF_INT, # Output, irank.
    utils._EngineFloat64ArrayType.c_type, # Output, p[2*ip+ip*ip].
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Output, wk[max(2, 5*(ip-1)+ip*ip)].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

linregm_obs_edit = utils._EngineState._get_symbol('g02dcf')
linregm_obs_edit.restype = None
linregm_obs_edit.argtypes = (
    utils._BYREF_STRING, # Input, update(len=1).
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, m.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, ip+1].
    utils._BYREF_INT, # Input, ldq.
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, x[*].
    utils._BYREF_INT, # Input, ix.
    utils._BYREF_FLOAT64, # Input, y.
    utils._BYREF_FLOAT64, # Input, wt.
    utils._BYREF_FLOAT64, # Input/Output, rss.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[3*ip].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of update.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

linregm_update = utils._EngineState._get_symbol('g02ddf')
linregm_update.restype = None
linregm_update.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, q[ldq, ip+1].
    utils._BYREF_INT, # Input, ldq.
    utils._BYREF_FLOAT64, # Input/Output, rss.
    utils._BYREF_INT, # Output, idf.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, cov[ip*(ip+1)/2].
    utils._BYREF_LOGICAL, # Output, svd.
    utils._BYREF_INT, # Output, irank.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, p[ip*ip+2*ip].
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[ip*ip+(ip-1)*5].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_var_add = utils._EngineState._get_symbol('g02def')
linregm_var_add.restype = None
linregm_var_add.argtypes = (
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, ip+2].
    utils._BYREF_INT, # Input, ldq.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, p[ip+1].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._BYREF_FLOAT64, # Output, rss.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

linregm_var_del = utils._EngineState._get_symbol('g02dff')
linregm_var_del.restype = None
linregm_var_del.argtypes = (
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, ip+1].
    utils._BYREF_INT, # Input, ldq.
    utils._BYREF_INT, # Input, indx.
    utils._BYREF_FLOAT64, # Input/Output, rss.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[2*ip].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_fit_newvar = utils._EngineState._get_symbol('g02dgf')
linregm_fit_newvar.restype = None
linregm_fit_newvar.argtypes = (
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_FLOAT64, # Input/Output, rss.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, irank.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, ip+1].
    utils._BYREF_INT, # Input, ldq.
    utils._BYREF_LOGICAL, # Input, svd.
    utils._EngineFloat64ArrayType.c_type, # Input, p[*].
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Output, b[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, res[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wk[5*(ip-1)+ip*ip].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

linregm_constrain = utils._EngineState._get_symbol('g02dkf')
linregm_constrain.restype = None
linregm_constrain.argtypes = (
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iconst.
    utils._EngineFloat64ArrayType.c_type, # Input, p[ip*ip+2*ip].
    utils._EngineFloat64ArrayType.c_type, # Input, c[ldc, iconst].
    utils._BYREF_INT, # Input, ldc.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ip].
    utils._BYREF_FLOAT64, # Input, rss.
    utils._BYREF_INT, # Input, idf.
    utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[2*ip*ip+ip*iconst+2*iconst*iconst+4*iconst].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_estfunc = utils._EngineState._get_symbol('g02dnf')
linregm_estfunc.restype = None
linregm_estfunc.argtypes = (
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, irank.
    utils._EngineFloat64ArrayType.c_type, # Input, b[ip].
    utils._EngineFloat64ArrayType.c_type, # Input, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Input, p[ip*ip+2*ip].
    utils._EngineFloat64ArrayType.c_type, # Input, f[ip].
    utils._BYREF_LOGICAL, # Output, est.
    utils._BYREF_FLOAT64, # Output, stat.
    utils._BYREF_FLOAT64, # Output, sestat.
    utils._BYREF_FLOAT64, # Output, t.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[ip].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_rssq = utils._EngineState._get_symbol('g02eaf')
linregm_rssq.restype = None
linregm_rssq.argtypes = (
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineCharArrayType.c_type, # Input, vname(len=*)[m].
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_INT, # Output, nmod.
    utils._EngineCharArrayType.c_type, # Output, modl(len=*)[ldmodl, m].
    utils._BYREF_INT, # Input, ldmodl.
    utils._EngineFloat64ArrayType.c_type, # Output, rss[ldmodl].
    utils._EngineIntArrayType.c_type, # Output, nterms[ldmodl].
    utils._EngineIntArrayType.c_type, # Output, mrank[ldmodl].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[n*(m+1)].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vname.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of modl.
)

linregm_rssq_stat = utils._EngineState._get_symbol('g02ecf')
linregm_rssq_stat.restype = None
linregm_rssq_stat.argtypes = (
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Input, sigsq.
    utils._BYREF_FLOAT64, # Input, tss.
    utils._BYREF_INT, # Input, nmod.
    utils._EngineIntArrayType.c_type, # Input, nterms[nmod].
    utils._EngineFloat64ArrayType.c_type, # Input, rss[nmod].
    utils._EngineFloat64ArrayType.c_type, # Output, rsq[nmod].
    utils._EngineFloat64ArrayType.c_type, # Output, cp[nmod].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
)

linregm_fit_onestep = utils._EngineState._get_symbol('g02eef')
linregm_fit_onestep.restype = None
linregm_fit_onestep.argtypes = (
    utils._BYREF_INT, # Input/Output, istep.
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineCharArrayType.c_type, # Input, vname(len=*)[m].
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, maxip.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_FLOAT64, # Input, fin.
    utils._BYREF_LOGICAL, # Output, addvar.
    utils._BYREF_STRING, # Output, newvar(len=*).
    utils._BYREF_FLOAT64, # Output, chrss.
    utils._BYREF_FLOAT64, # Output, f.
    utils._EngineCharArrayType.c_type, # Input/Output, model(len=*)[maxip].
    utils._BYREF_INT, # Input/Output, nterm.
    utils._BYREF_FLOAT64, # Input/Output, rss.
    utils._BYREF_INT, # Input/Output, idf.
    utils._BYREF_INT, # Input/Output, ifr.
    utils._EngineCharArrayType.c_type, # Input/Output, free(len=*)[maxip].
    utils._EngineFloat64ArrayType.c_type, # Output, exss[maxip].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, q[ldq, maxip+2].
    utils._BYREF_INT, # Input, ldq.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, p[maxip+1].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[2*maxip].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of vname.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of newvar.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of model.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of free.
)

linregm_fit_stepwise_monfun = utils._EngineState._sys_functype(
    None,
    utils._BYREF_STRING, # Input, flag(len=1).
    utils._BYREF_INT, # Input, var.
    utils._BYREF_FLOAT64, # Input, val.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of flag.
)
linregm_fit_stepwise = utils._EngineState._get_symbol('g02eff')
linregm_fit_stepwise.restype = None
linregm_fit_stepwise.argtypes = (
    utils._BYREF_INT, # Input, m.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, wmean[m+1].
    utils._EngineFloat64ArrayType.c_type, # Input, c[(m+1)*(m+2)/2].
    utils._BYREF_FLOAT64, # Input, sw.
    utils._EngineIntArrayType.c_type, # Input/Output, isx[m].
    utils._BYREF_FLOAT64, # Input, fin.
    utils._BYREF_FLOAT64, # Input, fout.
    utils._BYREF_FLOAT64, # Input, tau.
    utils._EngineFloat64ArrayType.c_type, # Output, b[m+1].
    utils._EngineFloat64ArrayType.c_type, # Output, se[m+1].
    utils._BYREF_FLOAT64, # Output, rsq.
    utils._BYREF_FLOAT64, # Output, rms.
    utils._BYREF_INT, # Output, df.
    utils._BYREF_INT, # Input, monlev.
    linregm_fit_stepwise_monfun,
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_fit_stepwise_sample_monfun = utils._EngineState._get_symbol('g02efh')
linregm_fit_stepwise_sample_monfun.restype = None
linregm_fit_stepwise_sample_monfun.argtypes = (
    utils._BYREF_STRING, # Input, flag(len=1).
    utils._BYREF_INT, # Input, var.
    utils._BYREF_FLOAT64, # Input, val.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of flag.
)

linregm_stat_resinf = utils._EngineState._get_symbol('g02faf')
linregm_stat_resinf.restype = None
linregm_stat_resinf.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, nres.
    utils._EngineFloat64ArrayType.c_type, # Input, res[nres].
    utils._EngineFloat64ArrayType.c_type, # Input, h[nres].
    utils._BYREF_FLOAT64, # Input, rms.
    utils._EngineFloat64ArrayType.c_type, # Output, sres[ldsres, 4].
    utils._BYREF_INT, # Input, ldsres.
    utils._BYREF_INT, # Input/Output, ifail.
)

linregm_stat_durbwat = utils._EngineState._get_symbol('g02fcf')
linregm_stat_durbwat.restype = None
linregm_stat_durbwat.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, res[n].
    utils._BYREF_FLOAT64, # Output, d.
    utils._BYREF_FLOAT64, # Output, pdl.
    utils._BYREF_FLOAT64, # Output, pdu.
    utils._EngineFloat64ArrayType.c_type, # Workspace, work[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

glm_normal = utils._EngineState._get_symbol('g02gaf')
glm_normal.restype = None
glm_normal.argtypes = (
    utils._BYREF_STRING, # Input, link(len=1).
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, offset(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, m.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_FLOAT64, # Input/Output, s.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Output, rss.
    utils._BYREF_INT, # Output, idf.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ip].
    utils._BYREF_INT, # Output, irank.
    utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, ip+7].
    utils._BYREF_INT, # Input, ldv.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, iprint.
    utils._BYREF_FLOAT64, # Input, eps.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[(ip*ip+3*ip+22)/2].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of link.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of offset.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

glm_binomial = utils._EngineState._get_symbol('g02gbf')
glm_binomial.restype = None
glm_binomial.argtypes = (
    utils._BYREF_STRING, # Input, link(len=1).
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, offset(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, m.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, t[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_FLOAT64, # Output, dev.
    utils._BYREF_INT, # Output, idf.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ip].
    utils._BYREF_INT, # Output, irank.
    utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, ip+7].
    utils._BYREF_INT, # Input, ldv.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, iprint.
    utils._BYREF_FLOAT64, # Input, eps.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[(ip*ip+3*ip+22)/2].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of link.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of offset.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

glm_poisson = utils._EngineState._get_symbol('g02gcf')
glm_poisson.restype = None
glm_poisson.argtypes = (
    utils._BYREF_STRING, # Input, link(len=1).
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, offset(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, m.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Output, dev.
    utils._BYREF_INT, # Output, idf.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ip].
    utils._BYREF_INT, # Output, irank.
    utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, ip+7].
    utils._BYREF_INT, # Input, ldv.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, iprint.
    utils._BYREF_FLOAT64, # Input, eps.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[(ip*ip+3*ip+22)/2].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of link.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of offset.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

glm_gamma = utils._EngineState._get_symbol('g02gdf')
glm_gamma.restype = None
glm_gamma.argtypes = (
    utils._BYREF_STRING, # Input, link(len=1).
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, offset(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, m.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_FLOAT64, # Input/Output, s.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Output, dev.
    utils._BYREF_INT, # Output, idf.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ip].
    utils._BYREF_INT, # Output, irank.
    utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, v[ldv, ip+7].
    utils._BYREF_INT, # Input, ldv.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, iprint.
    utils._BYREF_FLOAT64, # Input, eps.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[(ip*ip+3*ip+22)/2].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of link.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of offset.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

glm_constrain = utils._EngineState._get_symbol('g02gkf')
glm_constrain.restype = None
glm_constrain.argtypes = (
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, iconst.
    utils._EngineFloat64ArrayType.c_type, # Input, v[ldv, ip+7].
    utils._BYREF_INT, # Input, ldv.
    utils._EngineFloat64ArrayType.c_type, # Input, c[ldc, iconst].
    utils._BYREF_INT, # Input, ldc.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ip].
    utils._BYREF_FLOAT64, # Input, s.
    utils._EngineFloat64ArrayType.c_type, # Output, se[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[2*ip*ip+ip*iconst+2*iconst*iconst+4*iconst].
    utils._BYREF_INT, # Input/Output, ifail.
)

glm_estfunc = utils._EngineState._get_symbol('g02gnf')
glm_estfunc.restype = None
glm_estfunc.argtypes = (
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, irank.
    utils._EngineFloat64ArrayType.c_type, # Input, b[ip].
    utils._EngineFloat64ArrayType.c_type, # Input, cov[ip*(ip+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Input, v[ldv, ip+7].
    utils._BYREF_INT, # Input, ldv.
    utils._EngineFloat64ArrayType.c_type, # Input, f[ip].
    utils._BYREF_LOGICAL, # Output, est.
    utils._BYREF_FLOAT64, # Output, stat.
    utils._BYREF_FLOAT64, # Output, sestat.
    utils._BYREF_FLOAT64, # Output, z.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[ip].
    utils._BYREF_INT, # Input/Output, ifail.
)

glm_predict = utils._EngineState._get_symbol('g02gpf')
glm_predict.restype = None
glm_predict.argtypes = (
    utils._BYREF_STRING, # Input, errfn(len=1).
    utils._BYREF_STRING, # Input, link(len=1).
    utils._BYREF_STRING, # Input, mean(len=1).
    utils._BYREF_STRING, # Input, offset(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, *].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_INT, # Input, m.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, t[*].
    utils._EngineFloat64ArrayType.c_type, # Input, off[*].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_FLOAT64, # Input, s.
    utils._BYREF_FLOAT64, # Input, a.
    utils._EngineFloat64ArrayType.c_type, # Input, b[ip].
    utils._EngineFloat64ArrayType.c_type, # Input, cov[ip*(ip+1)/2].
    utils._BYREF_LOGICAL, # Input, vfobs.
    utils._EngineFloat64ArrayType.c_type, # Output, eta[n].
    utils._EngineFloat64ArrayType.c_type, # Output, seeta[n].
    utils._EngineFloat64ArrayType.c_type, # Output, pred[n].
    utils._EngineFloat64ArrayType.c_type, # Output, sepred[n].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errfn.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of link.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of offset.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

robustm = utils._EngineState._get_symbol('g02haf')
robustm.restype = None
robustm.argtypes = (
    utils._BYREF_INT, # Input, indw.
    utils._BYREF_INT, # Input, ipsi.
    utils._BYREF_INT, # Input, isigma.
    utils._BYREF_INT, # Input, indc.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
    utils._BYREF_FLOAT64, # Input, cpsi.
    utils._BYREF_FLOAT64, # Input, h1.
    utils._BYREF_FLOAT64, # Input, h2.
    utils._BYREF_FLOAT64, # Input, h3.
    utils._BYREF_FLOAT64, # Input, cucv.
    utils._BYREF_FLOAT64, # Input, dchi.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, theta[m].
    utils._BYREF_FLOAT64, # Input/Output, sigma.
    utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, m].
    utils._BYREF_INT, # Input, ldc.
    utils._EngineFloat64ArrayType.c_type, # Output, rs[n].
    utils._EngineFloat64ArrayType.c_type, # Output, wgt[n].
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, nitmon.
    utils._EngineFloat64ArrayType.c_type, # Output, stat[4].
    utils._BYREF_INT, # Input/Output, ifail.
)

robustm_wts_ucv = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, t.
)
robustm_wts = utils._EngineState._get_symbol('g02hbf')
robustm_wts.restype = None
robustm_wts.argtypes = (
    robustm_wts_ucv,
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, a[m*(m+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Output, z[n].
    utils._BYREF_FLOAT64, # Input, bl.
    utils._BYREF_FLOAT64, # Input, bd.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, nitmon.
    utils._BYREF_INT, # Output, nit.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[m*(m+1)/2].
    utils._BYREF_INT, # Input/Output, ifail.
)

robustm_user_chi = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, t.
)
robustm_user_psi = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, t.
)
robustm_user = utils._EngineState._get_symbol('g02hdf')
robustm_user.restype = None
robustm_user.argtypes = (
    robustm_user_chi,
    robustm_user_psi,
    utils._BYREF_FLOAT64, # Input, psip0.
    utils._BYREF_FLOAT64, # Input, beta.
    utils._BYREF_INT, # Input, indw.
    utils._BYREF_INT, # Input, isigma.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, wgt[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, theta[m].
    utils._BYREF_INT, # Output, k.
    utils._BYREF_FLOAT64, # Input/Output, sigma.
    utils._EngineFloat64ArrayType.c_type, # Output, rs[n].
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_FLOAT64, # Input, eps.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, nitmon.
    utils._BYREF_INT, # Output, nit.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[(m+4)*n].
    utils._BYREF_INT, # Input/Output, ifail.
)

robustm_user_dummy_chi = utils._EngineState._get_symbol('g02hdz')
robustm_user_dummy_chi.restype = utils._EngineFloat64ScalarType.c_type
robustm_user_dummy_chi.argtypes = (
    utils._BYREF_FLOAT64, # Input, t.
)

robustm_user_varmat_psi = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, t.
)
robustm_user_varmat_psp = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, t.
)
robustm_user_varmat = utils._EngineState._get_symbol('g02hff')
robustm_user_varmat.restype = None
robustm_user_varmat.argtypes = (
    robustm_user_varmat_psi,
    robustm_user_varmat_psp,
    utils._BYREF_INT, # Input, indw.
    utils._BYREF_INT, # Input, indc.
    utils._BYREF_FLOAT64, # Input, sigma.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Input, rs[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wgt[n].
    utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, m].
    utils._BYREF_INT, # Input, ldc.
    utils._EngineFloat64ArrayType.c_type, # Output, wk[m*(n+m+1)+2*n].
    utils._BYREF_INT, # Input/Output, ifail.
)

robustm_corr_huber = utils._EngineState._get_symbol('g02hkf')
robustm_corr_huber.restype = None
robustm_corr_huber.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._BYREF_FLOAT64, # Input, eps.
    utils._EngineFloat64ArrayType.c_type, # Output, cov[m*(m+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Output, theta[m].
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, nitmon.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Output, nit.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[n+m*(m+5)/2].
    utils._BYREF_INT, # Input/Output, ifail.
)

robustm_corr_user_deriv_ucv = utils._EngineState._sys_functype(
    None,
    utils._BYREF_FLOAT64, # Input, t.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_FLOAT64, # Output, u.
    utils._BYREF_FLOAT64, # Output, ud.
    utils._BYREF_FLOAT64, # Output, w.
    utils._BYREF_FLOAT64, # Output, wd.
)
robustm_corr_user_deriv = utils._EngineState._get_symbol('g02hlf')
robustm_corr_user_deriv.restype = None
robustm_corr_user_deriv.argtypes = (
    robustm_corr_user_deriv_ucv,
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input, indm.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Output, cov[m*(m+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, a[m*(m+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Output, wt[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, theta[m].
    utils._BYREF_FLOAT64, # Input, bl.
    utils._BYREF_FLOAT64, # Input, bd.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, nitmon.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Output, nit.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[2*m].
    utils._BYREF_INT, # Input/Output, ifail.
)

robustm_corr_user_ucv = utils._EngineState._sys_functype(
    None,
    utils._BYREF_FLOAT64, # Input, t.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_FLOAT64, # Output, u.
    utils._BYREF_FLOAT64, # Output, w.
)
robustm_corr_user = utils._EngineState._get_symbol('g02hmf')
robustm_corr_user.restype = None
robustm_corr_user.argtypes = (
    robustm_corr_user_ucv,
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input, indm.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineFloat64ArrayType.c_type, # Output, cov[m*(m+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, a[m*(m+1)/2].
    utils._EngineFloat64ArrayType.c_type, # Output, wt[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, theta[m].
    utils._BYREF_FLOAT64, # Input, bl.
    utils._BYREF_FLOAT64, # Input, bd.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_INT, # Input, nitmon.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Output, nit.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wk[2*m].
    utils._BYREF_INT, # Input/Output, ifail.
)

mixeff_reml = utils._EngineState._get_symbol('g02jaf')
mixeff_reml.restype = None
mixeff_reml.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, ncol.
    utils._BYREF_INT, # Input, lddat.
    utils._EngineFloat64ArrayType.c_type, # Input, dat[lddat, ncol].
    utils._EngineIntArrayType.c_type, # Input, levels[ncol].
    utils._BYREF_INT, # Input, yvid.
    utils._BYREF_INT, # Input, cwid.
    utils._BYREF_INT, # Input, nfv.
    utils._EngineIntArrayType.c_type, # Input, fvid[nfv].
    utils._BYREF_INT, # Input, fint.
    utils._BYREF_INT, # Input, nrv.
    utils._EngineIntArrayType.c_type, # Input, rvid[nrv].
    utils._BYREF_INT, # Input, nvpr.
    utils._EngineIntArrayType.c_type, # Input, vpr[nrv].
    utils._BYREF_INT, # Input, rint.
    utils._BYREF_INT, # Input, svid.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, gamma[nvpr+2].
    utils._BYREF_INT, # Output, nff.
    utils._BYREF_INT, # Output, nrf.
    utils._BYREF_INT, # Output, df.
    utils._BYREF_FLOAT64, # Output, reml.
    utils._BYREF_INT, # Input, lb.
    utils._EngineFloat64ArrayType.c_type, # Output, b[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, se[lb].
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Output, warn.
    utils._BYREF_INT, # Input/Output, ifail.
)

mixeff_ml = utils._EngineState._get_symbol('g02jbf')
mixeff_ml.restype = None
mixeff_ml.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, ncol.
    utils._BYREF_INT, # Input, lddat.
    utils._EngineFloat64ArrayType.c_type, # Input, dat[lddat, ncol].
    utils._EngineIntArrayType.c_type, # Input, levels[ncol].
    utils._BYREF_INT, # Input, yvid.
    utils._BYREF_INT, # Input, cwid.
    utils._BYREF_INT, # Input, nfv.
    utils._EngineIntArrayType.c_type, # Input, fvid[nfv].
    utils._BYREF_INT, # Input, fint.
    utils._BYREF_INT, # Input, nrv.
    utils._EngineIntArrayType.c_type, # Input, rvid[nrv].
    utils._BYREF_INT, # Input, nvpr.
    utils._EngineIntArrayType.c_type, # Input, vpr[nrv].
    utils._BYREF_INT, # Input, rint.
    utils._BYREF_INT, # Input, svid.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, gamma[nvpr+2].
    utils._BYREF_INT, # Output, nff.
    utils._BYREF_INT, # Output, nrf.
    utils._BYREF_INT, # Output, df.
    utils._BYREF_FLOAT64, # Output, ml.
    utils._BYREF_INT, # Input, lb.
    utils._EngineFloat64ArrayType.c_type, # Output, b[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, se[lb].
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_INT, # Output, warn.
    utils._BYREF_INT, # Input/Output, ifail.
)

mixeff_hier_init = utils._EngineState._get_symbol('g02jcf')
mixeff_hier_init.restype = None
mixeff_hier_init.argtypes = (
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, ncol.
    utils._EngineFloat64ArrayType.c_type, # Input, dat[lddat, *].
    utils._BYREF_INT, # Input, lddat.
    utils._EngineIntArrayType.c_type, # Input, levels[ncol].
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._EngineIntArrayType.c_type, # Input, fixed[lfixed].
    utils._BYREF_INT, # Input, lfixed.
    utils._BYREF_INT, # Input, nrndm.
    utils._EngineIntArrayType.c_type, # Input, rndm[ldrndm, *].
    utils._BYREF_INT, # Input, ldrndm.
    utils._BYREF_INT, # Output, nff.
    utils._BYREF_INT, # Output, nlsv.
    utils._BYREF_INT, # Output, nrf.
    utils._EngineFloat64ArrayType.c_type, # Output, rcomm[lrcomm].
    utils._BYREF_INT, # Input, lrcomm.
    utils._EngineIntArrayType.c_type, # Output, icomm[licomm].
    utils._BYREF_INT, # Input, licomm.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

mixeff_hier_reml = utils._EngineState._get_symbol('g02jdf')
mixeff_hier_reml.restype = None
mixeff_hier_reml.argtypes = (
    utils._BYREF_INT, # Input, lvpr.
    utils._EngineIntArrayType.c_type, # Input, vpr[lvpr].
    utils._BYREF_INT, # Input, nvpr.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, gamma[nvpr+1].
    utils._BYREF_INT, # Output, effn.
    utils._BYREF_INT, # Output, rnkx.
    utils._BYREF_INT, # Output, ncov.
    utils._BYREF_FLOAT64, # Output, lnlike.
    utils._BYREF_INT, # Input, lb.
    utils._EngineIntArrayType.c_type, # Output, id[ldid, *].
    utils._BYREF_INT, # Input, ldid.
    utils._EngineFloat64ArrayType.c_type, # Output, b[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, se[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, czz[ldczz, *].
    utils._BYREF_INT, # Input, ldczz.
    utils._EngineFloat64ArrayType.c_type, # Output, cxx[ldcxx, *].
    utils._BYREF_INT, # Input, ldcxx.
    utils._EngineFloat64ArrayType.c_type, # Output, cxz[ldcxz, *].
    utils._BYREF_INT, # Input, ldcxz.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[*].
    utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
    utils._EngineIntArrayType.c_type, # Input, iopt[liopt].
    utils._BYREF_INT, # Input, liopt.
    utils._EngineFloat64ArrayType.c_type, # Input, ropt[lropt].
    utils._BYREF_INT, # Input, lropt.
    utils._BYREF_INT, # Input/Output, ifail.
)

mixeff_hier_ml = utils._EngineState._get_symbol('g02jef')
mixeff_hier_ml.restype = None
mixeff_hier_ml.argtypes = (
    utils._BYREF_INT, # Input, lvpr.
    utils._EngineIntArrayType.c_type, # Input, vpr[lvpr].
    utils._BYREF_INT, # Input, nvpr.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, gamma[nvpr+1].
    utils._BYREF_INT, # Output, effn.
    utils._BYREF_INT, # Output, rnkx.
    utils._BYREF_INT, # Output, ncov.
    utils._BYREF_FLOAT64, # Output, lnlike.
    utils._BYREF_INT, # Input, lb.
    utils._EngineIntArrayType.c_type, # Output, id[ldid, *].
    utils._BYREF_INT, # Input, ldid.
    utils._EngineFloat64ArrayType.c_type, # Output, b[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, se[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, czz[ldczz, *].
    utils._BYREF_INT, # Input, ldczz.
    utils._EngineFloat64ArrayType.c_type, # Output, cxx[ldcxx, *].
    utils._BYREF_INT, # Input, ldcxx.
    utils._EngineFloat64ArrayType.c_type, # Output, cxz[ldcxz, *].
    utils._BYREF_INT, # Input, ldcxz.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[*].
    utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
    utils._EngineIntArrayType.c_type, # Input, iopt[liopt].
    utils._BYREF_INT, # Input, liopt.
    utils._EngineFloat64ArrayType.c_type, # Input, ropt[lropt].
    utils._BYREF_INT, # Input, lropt.
    utils._BYREF_INT, # Input/Output, ifail.
)

lmm_init = utils._EngineState._get_symbol('g02jff')
lmm_init.restype = None
lmm_init.argtypes = (
    utils._BYREF_C_PTR, # Input/Output, hlmm.
    utils._BYREF_C_PTR, # Input, hddesc.
    utils._BYREF_C_PTR, # Input, hfixed.
    utils._BYREF_INT, # Input, nrndm.
    utils._EngineCptrArrayType.c_type, # Input, hrndm[nrndm].
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._EngineFloat64ArrayType.c_type, # Input, dat[lddat, sddat].
    utils._BYREF_INT, # Input, lddat.
    utils._BYREF_INT, # Input, sddat.
    utils._BYREF_INT, # Output, fnlsv.
    utils._BYREF_INT, # Output, nff.
    utils._BYREF_INT, # Output, rnlsv.
    utils._BYREF_INT, # Output, nrf.
    utils._BYREF_INT, # Output, nvpr.
    utils._EngineFloat64ArrayType.c_type, # Output, rcomm[lrcomm].
    utils._BYREF_INT, # Input, lrcomm.
    utils._EngineIntArrayType.c_type, # Output, icomm[licomm].
    utils._BYREF_INT, # Input, licomm.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

lmm_init_combine = utils._EngineState._get_symbol('g02jgf')
lmm_init_combine.restype = None
lmm_init_combine.argtypes = (
    utils._BYREF_C_PTR, # Input, hlmm.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xrcomm[xlrcomm].
    utils._BYREF_INT, # Input, xlrcomm.
    utils._EngineIntArrayType.c_type, # Input/Output, xicomm[xlicomm].
    utils._BYREF_INT, # Input, xlicomm.
    utils._EngineFloat64ArrayType.c_type, # Input, yrcomm[*].
    utils._EngineIntArrayType.c_type, # Input, yicomm[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

lmm_fit = utils._EngineState._get_symbol('g02jhf')
lmm_fit.restype = None
lmm_fit.argtypes = (
    utils._BYREF_C_PTR, # Input, hlmm.
    utils._BYREF_INT, # Input, nvpr.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, gamma[nvpr+1].
    utils._BYREF_INT, # Output, effn.
    utils._BYREF_INT, # Output, rnkx.
    utils._BYREF_INT, # Output, ncov.
    utils._BYREF_FLOAT64, # Output, lnlike.
    utils._BYREF_INT, # Input, lb.
    utils._EngineFloat64ArrayType.c_type, # Output, b[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, se[lb].
    utils._EngineFloat64ArrayType.c_type, # Output, czz[ldczz, *].
    utils._BYREF_INT, # Input, ldczz.
    utils._EngineFloat64ArrayType.c_type, # Output, cxx[ldcxx, *].
    utils._BYREF_INT, # Input, ldcxx.
    utils._EngineFloat64ArrayType.c_type, # Output, cxz[ldcxz, *].
    utils._BYREF_INT, # Input, ldcxz.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[*].
    utils._EngineIntArrayType.c_type, # Input/Output, icomm[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

ridge_opt = utils._EngineState._get_symbol('g02kaf')
ridge_opt.restype = None
ridge_opt.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_FLOAT64, # Input, tau.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_FLOAT64, # Input/Output, h.
    utils._BYREF_INT, # Input, opt.
    utils._BYREF_INT, # Input/Output, niter.
    utils._BYREF_FLOAT64, # Input, tol.
    utils._BYREF_FLOAT64, # Output, nep.
    utils._BYREF_INT, # Input, orig.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ip+1].
    utils._EngineFloat64ArrayType.c_type, # Output, vif[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, res[n].
    utils._BYREF_FLOAT64, # Output, rss.
    utils._BYREF_INT, # Output, df.
    utils._BYREF_INT, # Input, optloo.
    utils._EngineFloat64ArrayType.c_type, # Output, perr[5].
    utils._BYREF_INT, # Input/Output, ifail.
)

ridge = utils._EngineState._get_symbol('g02kbf')
ridge.restype = None
ridge.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, m].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_INT, # Input, lh.
    utils._EngineFloat64ArrayType.c_type, # Input, h[lh].
    utils._EngineFloat64ArrayType.c_type, # Output, nep[lh].
    utils._BYREF_INT, # Input, wantb.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ldb, *].
    utils._BYREF_INT, # Input, ldb.
    utils._BYREF_INT, # Input, wantvf.
    utils._EngineFloat64ArrayType.c_type, # Output, vf[ldvf, *].
    utils._BYREF_INT, # Input, ldvf.
    utils._BYREF_INT, # Input, lpec.
    utils._EngineCharArrayType.c_type, # Input, pec(len=1)[lpec].
    utils._EngineFloat64ArrayType.c_type, # Output, pe[ldpe, *].
    utils._BYREF_INT, # Input, ldpe.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of pec.
)

pls_svd = utils._EngineState._get_symbol('g02laf')
pls_svd.restype = None
pls_svd.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, mx.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, mx].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input, isx[mx].
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, my.
    utils._EngineFloat64ArrayType.c_type, # Input, y[ldy, my].
    utils._BYREF_INT, # Input, ldy.
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, ybar[my].
    utils._BYREF_INT, # Input, iscale.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xstd[ip].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ystd[my].
    utils._BYREF_INT, # Input, maxfac.
    utils._EngineFloat64ArrayType.c_type, # Output, xres[ldxres, ip].
    utils._BYREF_INT, # Input, ldxres.
    utils._EngineFloat64ArrayType.c_type, # Output, yres[ldyres, my].
    utils._BYREF_INT, # Input, ldyres.
    utils._EngineFloat64ArrayType.c_type, # Output, w[ldw, maxfac].
    utils._BYREF_INT, # Input, ldw.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, maxfac].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Output, t[ldt, maxfac].
    utils._BYREF_INT, # Input, ldt.
    utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, maxfac].
    utils._BYREF_INT, # Input, ldc.
    utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, maxfac].
    utils._BYREF_INT, # Input, ldu.
    utils._EngineFloat64ArrayType.c_type, # Output, xcv[maxfac].
    utils._EngineFloat64ArrayType.c_type, # Output, ycv[ldycv, my].
    utils._BYREF_INT, # Input, ldycv.
    utils._BYREF_INT, # Input/Output, ifail.
)

pls_wold = utils._EngineState._get_symbol('g02lbf')
pls_wold.restype = None
pls_wold.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, mx.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, mx].
    utils._BYREF_INT, # Input, ldx.
    utils._EngineIntArrayType.c_type, # Input, isx[mx].
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, my.
    utils._EngineFloat64ArrayType.c_type, # Input, y[ldy, my].
    utils._BYREF_INT, # Input, ldy.
    utils._EngineFloat64ArrayType.c_type, # Output, xbar[ip].
    utils._EngineFloat64ArrayType.c_type, # Output, ybar[my].
    utils._BYREF_INT, # Input, iscale.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, xstd[ip].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ystd[my].
    utils._BYREF_INT, # Input, maxfac.
    utils._BYREF_INT, # Input, maxit.
    utils._BYREF_FLOAT64, # Input, tau.
    utils._EngineFloat64ArrayType.c_type, # Output, xres[ldxres, ip].
    utils._BYREF_INT, # Input, ldxres.
    utils._EngineFloat64ArrayType.c_type, # Output, yres[ldyres, my].
    utils._BYREF_INT, # Input, ldyres.
    utils._EngineFloat64ArrayType.c_type, # Output, w[ldw, maxfac].
    utils._BYREF_INT, # Input, ldw.
    utils._EngineFloat64ArrayType.c_type, # Output, p[ldp, maxfac].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Output, t[ldt, maxfac].
    utils._BYREF_INT, # Input, ldt.
    utils._EngineFloat64ArrayType.c_type, # Output, c[ldc, maxfac].
    utils._BYREF_INT, # Input, ldc.
    utils._EngineFloat64ArrayType.c_type, # Output, u[ldu, maxfac].
    utils._BYREF_INT, # Input, ldu.
    utils._EngineFloat64ArrayType.c_type, # Output, xcv[maxfac].
    utils._EngineFloat64ArrayType.c_type, # Output, ycv[ldycv, my].
    utils._BYREF_INT, # Input, ldycv.
    utils._BYREF_INT, # Input/Output, ifail.
)

pls_fit = utils._EngineState._get_symbol('g02lcf')
pls_fit.restype = None
pls_fit.argtypes = (
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, my.
    utils._BYREF_INT, # Input, maxfac.
    utils._BYREF_INT, # Input, nfact.
    utils._EngineFloat64ArrayType.c_type, # Input, p[ldp, maxfac].
    utils._BYREF_INT, # Input, ldp.
    utils._EngineFloat64ArrayType.c_type, # Input, c[ldc, maxfac].
    utils._BYREF_INT, # Input, ldc.
    utils._EngineFloat64ArrayType.c_type, # Input, w[ldw, maxfac].
    utils._BYREF_INT, # Input, ldw.
    utils._BYREF_FLOAT64, # Input, rcond.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ldb, my].
    utils._BYREF_INT, # Input, ldb.
    utils._BYREF_INT, # Input, orig.
    utils._EngineFloat64ArrayType.c_type, # Input, xbar[ip].
    utils._EngineFloat64ArrayType.c_type, # Input, ybar[my].
    utils._BYREF_INT, # Input, iscale.
    utils._EngineFloat64ArrayType.c_type, # Input, xstd[ip].
    utils._EngineFloat64ArrayType.c_type, # Input, ystd[my].
    utils._EngineFloat64ArrayType.c_type, # Output, ob[ldob, my].
    utils._BYREF_INT, # Input, ldob.
    utils._BYREF_INT, # Input, vipopt.
    utils._EngineFloat64ArrayType.c_type, # Input, ycv[ldycv, my].
    utils._BYREF_INT, # Input, ldycv.
    utils._EngineFloat64ArrayType.c_type, # Output, vip[ldvip, vipopt].
    utils._BYREF_INT, # Input, ldvip.
    utils._BYREF_INT, # Input/Output, ifail.
)

pls_pred = utils._EngineState._get_symbol('g02ldf')
pls_pred.restype = None
pls_pred.argtypes = (
    utils._BYREF_INT, # Input, ip.
    utils._BYREF_INT, # Input, my.
    utils._BYREF_INT, # Input, orig.
    utils._EngineFloat64ArrayType.c_type, # Input, xbar[ip].
    utils._EngineFloat64ArrayType.c_type, # Input, ybar[my].
    utils._BYREF_INT, # Input, iscale.
    utils._EngineFloat64ArrayType.c_type, # Input, xstd[ip].
    utils._EngineFloat64ArrayType.c_type, # Input, ystd[my].
    utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, my].
    utils._BYREF_INT, # Input, ldb.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, mz.
    utils._EngineIntArrayType.c_type, # Input, isz[mz].
    utils._EngineFloat64ArrayType.c_type, # Input, z[ldz, mz].
    utils._BYREF_INT, # Input, ldz.
    utils._EngineFloat64ArrayType.c_type, # Output, yhat[ldyhat, my].
    utils._BYREF_INT, # Input, ldyhat.
    utils._BYREF_INT, # Input/Output, ifail.
)

lars = utils._EngineState._get_symbol('g02maf')
lars.restype = None
lars.argtypes = (
    utils._BYREF_INT, # Input, mtype.
    utils._BYREF_INT, # Input, pred.
    utils._BYREF_INT, # Input, prey.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, d[ldd, *].
    utils._BYREF_INT, # Input, ldd.
    utils._EngineIntArrayType.c_type, # Input, isx[lisx].
    utils._BYREF_INT, # Input, lisx.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_INT, # Input, mnstep.
    utils._BYREF_INT, # Output, ip.
    utils._BYREF_INT, # Output, nstep.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ldb, *].
    utils._BYREF_INT, # Input, ldb.
    utils._EngineFloat64ArrayType.c_type, # Output, fitsum[6, mnstep+1].
    utils._EngineFloat64ArrayType.c_type, # Input, ropt[lropt].
    utils._BYREF_INT, # Input, lropt.
    utils._BYREF_INT, # Input/Output, ifail.
)

lars_xtx = utils._EngineState._get_symbol('g02mbf')
lars_xtx.restype = None
lars_xtx.argtypes = (
    utils._BYREF_INT, # Input, mtype.
    utils._BYREF_INT, # Input, pred.
    utils._BYREF_INT, # Input, intcpt.
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, dtd[lddtd, *].
    utils._BYREF_INT, # Input, lddtd.
    utils._EngineIntArrayType.c_type, # Input, isx[lisx].
    utils._BYREF_INT, # Input, lisx.
    utils._EngineFloat64ArrayType.c_type, # Input, dty[m].
    utils._BYREF_FLOAT64, # Input, yty.
    utils._BYREF_INT, # Input, mnstep.
    utils._BYREF_INT, # Output, ip.
    utils._BYREF_INT, # Output, nstep.
    utils._EngineFloat64ArrayType.c_type, # Output, b[ldb, *].
    utils._BYREF_INT, # Input, ldb.
    utils._EngineFloat64ArrayType.c_type, # Output, fitsum[6, mnstep+1].
    utils._EngineFloat64ArrayType.c_type, # Input, ropt[lropt].
    utils._BYREF_INT, # Input, lropt.
    utils._BYREF_INT, # Input/Output, ifail.
)

lars_param = utils._EngineState._get_symbol('g02mcf')
lars_param.restype = None
lars_param.argtypes = (
    utils._BYREF_INT, # Input, nstep.
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, *].
    utils._BYREF_INT, # Input, ldb.
    utils._EngineFloat64ArrayType.c_type, # Input, fitsum[6, nstep+1].
    utils._BYREF_INT, # Input, ktype.
    utils._EngineFloat64ArrayType.c_type, # Input, nk[lnk].
    utils._BYREF_INT, # Input, lnk.
    utils._EngineFloat64ArrayType.c_type, # Output, nb[ldnb, *].
    utils._BYREF_INT, # Input, ldnb.
    utils._BYREF_INT, # Input/Output, ifail.
)

quantile_linreg_easy = utils._EngineState._get_symbol('g02qff')
quantile_linreg_easy.restype = None
quantile_linreg_easy.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, x[n, m].
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_INT, # Input, ntau.
    utils._EngineFloat64ArrayType.c_type, # Input, tau[ntau].
    utils._BYREF_FLOAT64, # Output, df.
    utils._EngineFloat64ArrayType.c_type, # Output, b[m, ntau].
    utils._EngineFloat64ArrayType.c_type, # Output, bl[m, ntau].
    utils._EngineFloat64ArrayType.c_type, # Output, bu[m, ntau].
    utils._EngineIntArrayType.c_type, # Output, info[ntau].
    utils._BYREF_INT, # Input/Output, ifail.
)

quantile_linreg = utils._EngineState._get_symbol('g02qgf')
quantile_linreg.restype = None
quantile_linreg.argtypes = (
    utils._BYREF_INT, # Input, sorder.
    utils._BYREF_STRING, # Input, intcpt(len=1).
    utils._BYREF_STRING, # Input, weight(len=1).
    utils._BYREF_INT, # Input, n.
    utils._BYREF_INT, # Input, m.
    utils._EngineFloat64ArrayType.c_type, # Input, dat[lddat, *].
    utils._BYREF_INT, # Input, lddat.
    utils._EngineIntArrayType.c_type, # Input, isx[m].
    utils._BYREF_INT, # Input, ip.
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._EngineFloat64ArrayType.c_type, # Input, wt[*].
    utils._BYREF_INT, # Input, ntau.
    utils._EngineFloat64ArrayType.c_type, # Input, tau[ntau].
    utils._BYREF_FLOAT64, # Output, df.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, b[ip, ntau].
    utils._EngineFloat64ArrayType.c_type, # Output, bl[ip, *].
    utils._EngineFloat64ArrayType.c_type, # Output, bu[ip, *].
    utils._EngineFloat64ArrayType.c_type, # Output, ch[ip, ip, *].
    utils._EngineFloat64ArrayType.c_type, # Output, res[n, *].
    utils._EngineIntArrayType.c_type, # Input, iopts[*].
    utils._EngineFloat64ArrayType.c_type, # Input, opts[*].
    utils._EngineIntArrayType.c_type, # Input/Output, state[*].
    utils._EngineIntArrayType.c_type, # Output, info[ntau].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of intcpt.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of weight.
)

optset = utils._EngineState._get_symbol('g02zkf')
optset.restype = None
optset.argtypes = (
    utils._BYREF_STRING, # Input, optstr(len=*).
    utils._EngineIntArrayType.c_type, # Input/Output, iopts[liopts].
    utils._BYREF_INT, # Input, liopts.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, opts[lopts].
    utils._BYREF_INT, # Input, lopts.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
)

optget = utils._EngineState._get_symbol('g02zlf')
optget.restype = None
optget.argtypes = (
    utils._BYREF_STRING, # Input, optstr(len=*).
    utils._BYREF_INT, # Output, ivalue.
    utils._BYREF_FLOAT64, # Output, rvalue.
    utils._BYREF_STRING, # Output, cvalue(len=*).
    utils._BYREF_INT, # Output, optype.
    utils._EngineIntArrayType.c_type, # Input, iopts[*].
    utils._EngineFloat64ArrayType.c_type, # Input, opts[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cvalue.
)
