# -*- coding: utf-8 -*-
# pylint: disable=invalid-name,line-too-long,protected-access,too-many-locals
# pylint: disable=too-many-lines
"""
Primitive ``ctypes`` prototypes for the NAG Library `quad` Chapter.
"""

# NAG Copyright 2017-2019.

from ..base import utils

dim1_fin_well_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_fin_well = utils._EngineState._get_symbol('d01ahf')
dim1_fin_well.restype = utils._EngineFloat64ScalarType.c_type
dim1_fin_well.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, epsr.
    utils._BYREF_INT, # Output, npts.
    utils._BYREF_FLOAT64, # Output, relerr.
    dim1_fin_well_f,
    utils._BYREF_INT, # Input, nlimit.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_bad_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_fin_bad = utils._EngineState._get_symbol('d01ajf')
dim1_fin_bad.restype = None
dim1_fin_bad.argtypes = (
    dim1_fin_bad_f,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_osc_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_fin_osc = utils._EngineState._get_symbol('d01akf')
dim1_fin_osc.restype = None
dim1_fin_osc.argtypes = (
    dim1_fin_osc_f,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_sing_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_fin_sing = utils._EngineState._get_symbol('d01alf')
dim1_fin_sing.restype = None
dim1_fin_sing.argtypes = (
    dim1_fin_sing_f,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_INT, # Input, npts.
    utils._EngineFloat64ArrayType.c_type, # Input, points[*].
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_inf_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_inf = utils._EngineState._get_symbol('d01amf')
dim1_inf.restype = None
dim1_inf.argtypes = (
    dim1_inf_f,
    utils._BYREF_FLOAT64, # Input, bound.
    utils._BYREF_INT, # Input, inf.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_wtrig_g = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_fin_wtrig = utils._EngineState._get_symbol('d01anf')
dim1_fin_wtrig.restype = None
dim1_fin_wtrig.argtypes = (
    dim1_fin_wtrig_g,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, omega.
    utils._BYREF_INT, # Input, key.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_wsing_g = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_fin_wsing = utils._EngineState._get_symbol('d01apf')
dim1_fin_wsing.restype = None
dim1_fin_wsing.argtypes = (
    dim1_fin_wsing_g,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, alfa.
    utils._BYREF_FLOAT64, # Input, beta.
    utils._BYREF_INT, # Input, key.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_wcauchy_g = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_fin_wcauchy = utils._EngineState._get_symbol('d01aqf')
dim1_fin_wcauchy.restype = None
dim1_fin_wcauchy.argtypes = (
    dim1_fin_wcauchy_g,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_indef_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_indef = utils._EngineState._get_symbol('d01arf')
dim1_indef.restype = None
dim1_indef.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    dim1_indef_f,
    utils._BYREF_FLOAT64, # Input, relacc.
    utils._BYREF_FLOAT64, # Input, absacc.
    utils._BYREF_INT, # Input, maxrul.
    utils._BYREF_INT, # Input, iparm.
    utils._BYREF_FLOAT64, # Output, acc.
    utils._BYREF_FLOAT64, # Output, ans.
    utils._BYREF_INT, # Output, n.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, alpha[390].
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_inf_wtrig_g = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_inf_wtrig = utils._EngineState._get_symbol('d01asf')
dim1_inf_wtrig.restype = None
dim1_inf_wtrig.argtypes = (
    dim1_inf_wtrig_g,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, omega.
    utils._BYREF_INT, # Input, key.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._BYREF_INT, # Input, limlst.
    utils._BYREF_INT, # Output, lst.
    utils._EngineFloat64ArrayType.c_type, # Output, erlst[limlst].
    utils._EngineFloat64ArrayType.c_type, # Output, rslst[limlst].
    utils._EngineIntArrayType.c_type, # Output, ierlst[limlst].
    utils._EngineFloat64ArrayType.c_type, # Workspace, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_bad_vec_f = utils._EngineState._sys_functype(
    None,
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, fv[n].
    utils._BYREF_INT, # Input, n.
)
dim1_fin_bad_vec = utils._EngineState._get_symbol('d01atf')
dim1_fin_bad_vec.restype = None
dim1_fin_bad_vec.argtypes = (
    dim1_fin_bad_vec_f,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_osc_vec_f = utils._EngineState._sys_functype(
    None,
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, fv[n].
    utils._BYREF_INT, # Input, n.
)
dim1_fin_osc_vec = utils._EngineState._get_symbol('d01auf')
dim1_fin_osc_vec.restype = None
dim1_fin_osc_vec.argtypes = (
    dim1_fin_osc_vec_f,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_INT, # Input, key.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
    utils._EngineFloat64ArrayType.c_type, # Output, w[lw].
    utils._BYREF_INT, # Input, lw.
    utils._EngineIntArrayType.c_type, # Output, iw[liw].
    utils._BYREF_INT, # Input, liw.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_gauss_wgen = utils._EngineState._get_symbol('d01bcf')
dim1_gauss_wgen.restype = None
dim1_gauss_wgen.argtypes = (
    utils._BYREF_INT, # Input, itype.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_FLOAT64, # Input, d.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Output, weight[n].
    utils._EngineFloat64ArrayType.c_type, # Output, abscis[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_smooth_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
)
dim1_fin_smooth = utils._EngineState._get_symbol('d01bdf')
dim1_fin_smooth.restype = None
dim1_fin_smooth.argtypes = (
    dim1_fin_smooth_f,
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, abserr.
)

dim2_fin_phi1 = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, y.
)
dim2_fin_phi2 = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, y.
)
dim2_fin_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_FLOAT64, # Input, x.
    utils._BYREF_FLOAT64, # Input, y.
)
dim2_fin = utils._EngineState._get_symbol('d01daf')
dim2_fin.restype = None
dim2_fin.argtypes = (
    utils._BYREF_FLOAT64, # Input, ya.
    utils._BYREF_FLOAT64, # Input, yb.
    dim2_fin_phi1,
    dim2_fin_phi2,
    dim2_fin_f,
    utils._BYREF_FLOAT64, # Input, absacc.
    utils._BYREF_FLOAT64, # Output, ans.
    utils._BYREF_INT, # Output, npts.
    utils._BYREF_INT, # Input/Output, ifail.
)

md_adapt_multi_funsub = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, z[ndim].
    utils._BYREF_INT, # Input, nfun.
    utils._EngineFloat64ArrayType.c_type, # Output, f[nfun].
)
md_adapt_multi = utils._EngineState._get_symbol('d01eaf')
md_adapt_multi.restype = None
md_adapt_multi.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, a[ndim].
    utils._EngineFloat64ArrayType.c_type, # Input, b[ndim].
    utils._BYREF_INT, # Input/Output, mincls.
    utils._BYREF_INT, # Input, maxcls.
    utils._BYREF_INT, # Input, nfun.
    md_adapt_multi_funsub,
    utils._BYREF_FLOAT64, # Input, absreq.
    utils._BYREF_FLOAT64, # Input, relreq.
    utils._BYREF_INT, # Input, lenwrk.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, wrkstr[lenwrk].
    utils._EngineFloat64ArrayType.c_type, # Output, finest[nfun].
    utils._EngineFloat64ArrayType.c_type, # Output, absest[nfun].
    utils._BYREF_INT, # Input/Output, ifail.
)

md_sgq_multi_vec_f = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, ni.
    utils._BYREF_INT, # Input, ndim.
    utils._BYREF_INT, # Input, nx.
    utils._BYREF_FLOAT64, # Input, xtr.
    utils._BYREF_INT, # Input, nntr.
    utils._EngineIntArrayType.c_type, # Input, icolzp[nx+1].
    utils._EngineIntArrayType.c_type, # Input, irowix[nntr].
    utils._EngineFloat64ArrayType.c_type, # Input, xs[nntr].
    utils._EngineIntArrayType.c_type, # Input, qs[nntr].
    utils._EngineFloat64ArrayType.c_type, # Output, fm[ni, nx].
    utils._BYREF_INT, # Input/Output, iflag.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
)
md_sgq_multi_vec = utils._EngineState._get_symbol('d01esf')
md_sgq_multi_vec.restype = None
md_sgq_multi_vec.argtypes = (
    utils._BYREF_INT, # Input, ni.
    utils._BYREF_INT, # Input, ndim.
    md_sgq_multi_vec_f,
    utils._EngineIntArrayType.c_type, # Input, maxdlv[ndim].
    utils._EngineFloat64ArrayType.c_type, # Output, dinest[ni].
    utils._EngineFloat64ArrayType.c_type, # Output, errest[ni].
    utils._EngineIntArrayType.c_type, # Output, ivalid[ni].
    utils._EngineIntArrayType.c_type, # Input, iopts[100].
    utils._EngineFloat64ArrayType.c_type, # Input, opts[100].
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

md_gauss_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
)
md_gauss = utils._EngineState._get_symbol('d01fbf')
md_gauss.restype = utils._EngineFloat64ScalarType.c_type
md_gauss.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._EngineIntArrayType.c_type, # Input, nptvec[ndim].
    utils._BYREF_INT, # Input, lwa.
    utils._EngineFloat64ArrayType.c_type, # Input, weight[lwa].
    utils._EngineFloat64ArrayType.c_type, # Input, abscis[lwa].
    md_gauss_f,
    utils._BYREF_INT, # Input/Output, ifail.
)

md_adapt_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, z[ndim].
)
md_adapt = utils._EngineState._get_symbol('d01fcf')
md_adapt.restype = None
md_adapt.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, a[ndim].
    utils._EngineFloat64ArrayType.c_type, # Input, b[ndim].
    utils._BYREF_INT, # Input/Output, minpts.
    utils._BYREF_INT, # Input, maxpts.
    md_adapt_f,
    utils._BYREF_FLOAT64, # Input, eps.
    utils._BYREF_FLOAT64, # Output, acc.
    utils._BYREF_INT, # Input, lenwrk.
    utils._EngineFloat64ArrayType.c_type, # Workspace, wrkstr[lenwrk].
    utils._BYREF_FLOAT64, # Output, finval.
    utils._BYREF_INT, # Input/Output, ifail.
)

md_sphere_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
)
md_sphere_region = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
    utils._BYREF_INT, # Input, j.
    utils._BYREF_FLOAT64, # Output, c.
    utils._BYREF_FLOAT64, # Output, d.
)
md_sphere = utils._EngineState._get_symbol('d01fdf')
md_sphere.restype = None
md_sphere.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    md_sphere_f,
    utils._BYREF_FLOAT64, # Input, sigma.
    md_sphere_region,
    utils._BYREF_INT, # Input, limit.
    utils._BYREF_FLOAT64, # Input, r0.
    utils._BYREF_FLOAT64, # Input, u.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_INT, # Output, ncalls.
    utils._BYREF_INT, # Input/Output, ifail.
)

md_sphere_dummy_region = utils._EngineState._get_symbol('d01fdv')
md_sphere_dummy_region.restype = None
md_sphere_dummy_region.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
    utils._BYREF_INT, # Input, j.
    utils._BYREF_FLOAT64, # Output, c.
    utils._BYREF_FLOAT64, # Output, d.
)

dim1_data = utils._EngineState._get_symbol('d01gaf')
dim1_data.restype = None
dim1_data.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Input, y[n].
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Output, ans.
    utils._BYREF_FLOAT64, # Output, er.
    utils._BYREF_INT, # Input/Output, ifail.
)

md_mcarlo_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
)
md_mcarlo = utils._EngineState._get_symbol('d01gbf')
md_mcarlo.restype = None
md_mcarlo.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, a[ndim].
    utils._EngineFloat64ArrayType.c_type, # Input, b[ndim].
    utils._BYREF_INT, # Input/Output, mincls.
    utils._BYREF_INT, # Input, maxcls.
    md_mcarlo_f,
    utils._BYREF_FLOAT64, # Input, eps.
    utils._BYREF_FLOAT64, # Output, acc.
    utils._BYREF_INT, # Input, lenwrk.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, wrkstr[lenwrk].
    utils._BYREF_FLOAT64, # Input/Output, finest.
    utils._BYREF_INT, # Input/Output, ifail.
)

md_numth_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
)
md_numth_region = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
    utils._BYREF_INT, # Input, j.
    utils._BYREF_FLOAT64, # Output, c.
    utils._BYREF_FLOAT64, # Output, d.
)
md_numth = utils._EngineState._get_symbol('d01gcf')
md_numth.restype = None
md_numth.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    md_numth_f,
    md_numth_region,
    utils._BYREF_INT, # Input, npts.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, vk[ndim].
    utils._BYREF_INT, # Input, nrand.
    utils._BYREF_INT, # Input, itrans.
    utils._BYREF_FLOAT64, # Output, res.
    utils._BYREF_FLOAT64, # Output, err.
    utils._BYREF_INT, # Input/Output, ifail.
)

md_numth_vec_vecfun = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m, ndim].
    utils._EngineFloat64ArrayType.c_type, # Output, fv[m].
    utils._BYREF_INT, # Input, m.
)
md_numth_vec_vecreg = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[m, ndim].
    utils._BYREF_INT, # Input, j.
    utils._EngineFloat64ArrayType.c_type, # Output, c[m].
    utils._EngineFloat64ArrayType.c_type, # Output, d[m].
    utils._BYREF_INT, # Input, m.
)
md_numth_vec = utils._EngineState._get_symbol('d01gdf')
md_numth_vec.restype = None
md_numth_vec.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    md_numth_vec_vecfun,
    md_numth_vec_vecreg,
    utils._BYREF_INT, # Input, npts.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, vk[ndim].
    utils._BYREF_INT, # Input, nrand.
    utils._BYREF_INT, # Input, itrans.
    utils._BYREF_FLOAT64, # Output, res.
    utils._BYREF_FLOAT64, # Output, err.
    utils._BYREF_INT, # Input/Output, ifail.
)

md_numth_coeff_prime = utils._EngineState._get_symbol('d01gyf')
md_numth_coeff_prime.restype = None
md_numth_coeff_prime.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._BYREF_INT, # Input, npts.
    utils._EngineFloat64ArrayType.c_type, # Output, vk[ndim].
    utils._BYREF_INT, # Input/Output, ifail.
)

md_numth_coeff_2prime = utils._EngineState._get_symbol('d01gzf')
md_numth_coeff_2prime.restype = None
md_numth_coeff_2prime.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._BYREF_INT, # Input, np1.
    utils._BYREF_INT, # Input, np2.
    utils._EngineFloat64ArrayType.c_type, # Output, vk[ndim].
    utils._BYREF_INT, # Input/Output, ifail.
)

md_sphere_bad_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
)
md_sphere_bad = utils._EngineState._get_symbol('d01jaf')
md_sphere_bad.restype = None
md_sphere_bad.argtypes = (
    md_sphere_bad_f,
    utils._BYREF_INT, # Input, ndim.
    utils._BYREF_FLOAT64, # Input, radius.
    utils._BYREF_FLOAT64, # Input, epsa.
    utils._BYREF_FLOAT64, # Input, epsr.
    utils._BYREF_INT, # Input, method.
    utils._BYREF_INT, # Input, icoord.
    utils._BYREF_FLOAT64, # Output, result.
    utils._BYREF_FLOAT64, # Output, esterr.
    utils._BYREF_INT, # Output, nevals.
    utils._BYREF_INT, # Input/Output, ifail.
)

md_simplex_f = utils._EngineState._sys_functype(
    utils._EngineFloat64ScalarType.c_type,
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input, x[ndim].
)
md_simplex = utils._EngineState._get_symbol('d01paf')
md_simplex.restype = None
md_simplex.argtypes = (
    utils._BYREF_INT, # Input, ndim.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, vert[ldvert, sdvert].
    utils._BYREF_INT, # Input, ldvert.
    utils._BYREF_INT, # Input, sdvert.
    md_simplex_f,
    utils._BYREF_INT, # Input/Output, minord.
    utils._BYREF_INT, # Input, maxord.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, finvls[maxord].
    utils._BYREF_FLOAT64, # Output, esterr.
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_gen_vec_multi_rcomm = utils._EngineState._get_symbol('d01raf')
dim1_gen_vec_multi_rcomm.restype = None
dim1_gen_vec_multi_rcomm.argtypes = (
    utils._BYREF_INT, # Input/Output, irevcm.
    utils._BYREF_INT, # Input, ni.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_INT, # Output, sid.
    utils._EngineIntArrayType.c_type, # Input/Output, needi[ni].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, x[lenx].
    utils._BYREF_INT, # Input, lenx.
    utils._BYREF_INT, # Input/Output, nx.
    utils._EngineFloat64ArrayType.c_type, # Input, fm[ldfm, *].
    utils._BYREF_INT, # Input, ldfm.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, dinest[ni].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, errest[ni].
    utils._EngineIntArrayType.c_type, # Input, iopts[100].
    utils._EngineFloat64ArrayType.c_type, # Input, opts[100].
    utils._EngineIntArrayType.c_type, # Input/Output, icom[licom].
    utils._BYREF_INT, # Input, licom.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, com[lcom].
    utils._BYREF_INT, # Input, lcom.
    utils._BYREF_INT, # Input/Output, ifail.
)

withdraw_1d_gen_vec_multi_diagnostic_monit = utils._EngineState._sys_functype(
    None,
    utils._BYREF_INT, # Input, ni.
    utils._BYREF_INT, # Input, ns.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, dinest[ni].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, errest[ni].
    utils._EngineIntArrayType.c_type, # Input, fcount[ni].
    utils._EngineIntArrayType.c_type, # Input, sinfoi[ldi, *].
    utils._EngineIntArrayType.c_type, # Input, evals[ldi, *].
    utils._BYREF_INT, # Input, ldi.
    utils._EngineFloat64ArrayType.c_type, # Input, sinfor[ldr, *].
    utils._EngineFloat64ArrayType.c_type, # Input, fs[ldr, *].
    utils._EngineFloat64ArrayType.c_type, # Input, es[ldr, *].
    utils._BYREF_INT, # Input, ldr.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
)
withdraw_1d_gen_vec_multi_diagnostic = utils._EngineState._get_symbol('d01rbf')
withdraw_1d_gen_vec_multi_diagnostic.restype = None
withdraw_1d_gen_vec_multi_diagnostic.argtypes = (
    withdraw_1d_gen_vec_multi_diagnostic_monit,
    utils._BYREF_INT, # Input, ni.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, dinest[ni].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, errest[ni].
    utils._EngineIntArrayType.c_type, # Input, icom[licom].
    utils._BYREF_INT, # Input, licom.
    utils._BYREF_INT, # Output, licusd.
    utils._EngineFloat64ArrayType.c_type, # Input, com[lcom].
    utils._BYREF_INT, # Input, lcom.
    utils._BYREF_INT, # Output, lcusd.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

d01rb_dummy = utils._EngineState._get_symbol('d01rbm')
d01rb_dummy.restype = None
d01rb_dummy.argtypes = (
    utils._BYREF_INT, # Input, ni.
    utils._BYREF_INT, # Input, ns.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, dinest[ni].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, errest[ni].
    utils._EngineIntArrayType.c_type, # Input, fcount[ni].
    utils._EngineIntArrayType.c_type, # Input, sinfoi[ldi, *].
    utils._EngineIntArrayType.c_type, # Input, evals[ldi, *].
    utils._BYREF_INT, # Input, ldi.
    utils._EngineFloat64ArrayType.c_type, # Input, sinfor[ldr, *].
    utils._EngineFloat64ArrayType.c_type, # Input, fs[ldr, *].
    utils._EngineFloat64ArrayType.c_type, # Input, es[ldr, *].
    utils._BYREF_INT, # Input, ldr.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
)

dim1_gen_vec_multi_dimreq = utils._EngineState._get_symbol('d01rcf')
dim1_gen_vec_multi_dimreq.restype = None
dim1_gen_vec_multi_dimreq.argtypes = (
    utils._BYREF_INT, # Input, ni.
    utils._BYREF_INT, # Output, lenxrq.
    utils._BYREF_INT, # Output, ldfmrq.
    utils._BYREF_INT, # Output, sdfmrq.
    utils._BYREF_INT, # Output, licmin.
    utils._BYREF_INT, # Output, licmax.
    utils._BYREF_INT, # Output, lcmin.
    utils._BYREF_INT, # Output, lcmax.
    utils._EngineIntArrayType.c_type, # Input, iopts[*].
    utils._EngineFloat64ArrayType.c_type, # Input, opts[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_fin_gonnet_vec_f = utils._EngineState._sys_functype(
    None,
    utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
    utils._BYREF_INT, # Input, nx.
    utils._EngineFloat64ArrayType.c_type, # Output, fv[nx].
    utils._BYREF_INT, # Input/Output, iflag.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
)
dim1_fin_gonnet_vec = utils._EngineState._get_symbol('d01rgf')
dim1_fin_gonnet_vec.restype = None
dim1_fin_gonnet_vec.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    dim1_fin_gonnet_vec_f,
    utils._BYREF_FLOAT64, # Input, epsabs.
    utils._BYREF_FLOAT64, # Input, epsrel.
    utils._BYREF_FLOAT64, # Output, dinest.
    utils._BYREF_FLOAT64, # Output, errest.
    utils._BYREF_INT, # Output, nevals.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_gauss_wres = utils._EngineState._get_symbol('d01tbf')
dim1_gauss_wres.restype = None
dim1_gauss_wres.argtypes = (
    utils._BYREF_INT, # Input, key.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Output, weight[n].
    utils._EngineFloat64ArrayType.c_type, # Output, abscis[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_gauss_wrec = utils._EngineState._get_symbol('d01tdf')
dim1_gauss_wrec.restype = None
dim1_gauss_wrec.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, a[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, b[n].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, c[n].
    utils._BYREF_FLOAT64, # Input, muzero.
    utils._EngineFloat64ArrayType.c_type, # Output, weight[n].
    utils._EngineFloat64ArrayType.c_type, # Output, abscis[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_gauss_recm = utils._EngineState._get_symbol('d01tef')
dim1_gauss_recm.restype = None
dim1_gauss_recm.argtypes = (
    utils._BYREF_INT, # Input, n.
    utils._EngineFloat64ArrayType.c_type, # Input, mu[2*n+1].
    utils._EngineFloat64ArrayType.c_type, # Output, a[n].
    utils._EngineFloat64ArrayType.c_type, # Output, b[n].
    utils._EngineFloat64ArrayType.c_type, # Output, c[n].
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_gauss_vec_f = utils._EngineState._sys_functype(
    None,
    utils._EngineFloat64ArrayType.c_type, # Input, x[nx].
    utils._BYREF_INT, # Input, nx.
    utils._EngineFloat64ArrayType.c_type, # Output, fv[nx].
    utils._BYREF_INT, # Input/Output, iflag.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
)
dim1_gauss_vec = utils._EngineState._get_symbol('d01uaf')
dim1_gauss_vec.restype = None
dim1_gauss_vec.argtypes = (
    utils._BYREF_INT, # Input, key.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_INT, # Input, n.
    dim1_gauss_vec_f,
    utils._BYREF_FLOAT64, # Output, dinest.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

dim1_inf_exp_wt_f = utils._EngineState._sys_functype(
    None,
    utils._EngineFloat64ArrayType.c_type, # Input, x[n].
    utils._EngineFloat64ArrayType.c_type, # Output, fv[n].
    utils._BYREF_INT, # Input, n.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, istop.
)
dim1_inf_exp_wt = utils._EngineState._get_symbol('d01ubf')
dim1_inf_exp_wt.restype = None
dim1_inf_exp_wt.argtypes = (
    dim1_inf_exp_wt_f,
    utils._BYREF_INT, # Input, n.
    utils._BYREF_FLOAT64, # Output, ans.
    utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
    utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    utils._BYREF_INT, # Input/Output, ifail.
)

opt_set = utils._EngineState._get_symbol('d01zkf')
opt_set.restype = None
opt_set.argtypes = (
    utils._BYREF_STRING, # Input, optstr(len=*).
    utils._EngineIntArrayType.c_type, # Input/Output, iopts[liopts].
    utils._BYREF_INT, # Input, liopts.
    utils._EngineFloat64ArrayType.c_type, # Input/Output, opts[lopts].
    utils._BYREF_INT, # Input, lopts.
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
)

opt_get = utils._EngineState._get_symbol('d01zlf')
opt_get.restype = None
opt_get.argtypes = (
    utils._BYREF_STRING, # Input, optstr(len=*).
    utils._BYREF_INT, # Output, ivalue.
    utils._BYREF_FLOAT64, # Output, rvalue.
    utils._BYREF_STRING, # Output, cvalue(len=*).
    utils._BYREF_INT, # Output, optype.
    utils._EngineIntArrayType.c_type, # Input, iopts[*].
    utils._EngineFloat64ArrayType.c_type, # Input, opts[*].
    utils._BYREF_INT, # Input/Output, ifail.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of optstr.
    utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cvalue.
)
