# -*- coding: utf-8 -*-
# pylint: disable=invalid-name,line-too-long,protected-access,too-many-locals
# pylint: disable=too-many-lines
"""
Primitive ``ctypes`` prototypes for the NAG Library `zeros` Chapter.
"""

# NAG Copyright 2017-2019.

from ..base import utils

poly_complex = utils._EngineState._get_symbol('c02aff')
poly_complex.restype = None
poly_complex.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, a[2, n+1].
    utils._BYREF_INT, # Input, n.
    utils._BYREF_LOGICAL, # Input, scal.
    utils._EngineFloat64ArrayType.c_type, # Output, z[2, n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, w[4*(n+1)].
    utils._BYREF_INT, # Input/Output, ifail.
)

poly_real = utils._EngineState._get_symbol('c02agf')
poly_real.restype = None
poly_real.argtypes = (
    utils._EngineFloat64ArrayType.c_type, # Input, a[n+1].
    utils._BYREF_INT, # Input, n.
    utils._BYREF_LOGICAL, # Input, scal.
    utils._EngineFloat64ArrayType.c_type, # Output, z[2, n].
    utils._EngineFloat64ArrayType.c_type, # Workspace, w[2*(n+1)].
    utils._BYREF_INT, # Input/Output, ifail.
)

quadratic_complex = utils._EngineState._get_symbol('c02ahf')
quadratic_complex.restype = None
quadratic_complex.argtypes = (
    utils._BYREF_FLOAT64, # Input, ar.
    utils._BYREF_FLOAT64, # Input, ai.
    utils._BYREF_FLOAT64, # Input, br.
    utils._BYREF_FLOAT64, # Input, bi.
    utils._BYREF_FLOAT64, # Input, cr.
    utils._BYREF_FLOAT64, # Input, ci.
    utils._EngineFloat64ArrayType.c_type, # Output, zsm[2].
    utils._EngineFloat64ArrayType.c_type, # Output, zlg[2].
    utils._BYREF_INT, # Input/Output, ifail.
)

quadratic_real = utils._EngineState._get_symbol('c02ajf')
quadratic_real.restype = None
quadratic_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, c.
    utils._EngineFloat64ArrayType.c_type, # Output, zsm[2].
    utils._EngineFloat64ArrayType.c_type, # Output, zlg[2].
    utils._BYREF_INT, # Input/Output, ifail.
)

cubic_real = utils._EngineState._get_symbol('c02akf')
cubic_real.restype = None
cubic_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, u.
    utils._BYREF_FLOAT64, # Input, r.
    utils._BYREF_FLOAT64, # Input, s.
    utils._BYREF_FLOAT64, # Input, t.
    utils._EngineFloat64ArrayType.c_type, # Output, zeror[3].
    utils._EngineFloat64ArrayType.c_type, # Output, zeroi[3].
    utils._EngineFloat64ArrayType.c_type, # Output, errest[3].
    utils._BYREF_INT, # Input/Output, ifail.
)

quartic_real = utils._EngineState._get_symbol('c02alf')
quartic_real.restype = None
quartic_real.argtypes = (
    utils._BYREF_FLOAT64, # Input, e.
    utils._BYREF_FLOAT64, # Input, a.
    utils._BYREF_FLOAT64, # Input, b.
    utils._BYREF_FLOAT64, # Input, c.
    utils._BYREF_FLOAT64, # Input, d.
    utils._EngineFloat64ArrayType.c_type, # Output, zeror[4].
    utils._EngineFloat64ArrayType.c_type, # Output, zeroi[4].
    utils._EngineFloat64ArrayType.c_type, # Output, errest[4].
    utils._BYREF_INT, # Input/Output, ifail.
)

cubic_complex = utils._EngineState._get_symbol('c02amf')
cubic_complex.restype = None
cubic_complex.argtypes = (
    utils._BYREF_COMPLEX128, # Input, u.
    utils._BYREF_COMPLEX128, # Input, r.
    utils._BYREF_COMPLEX128, # Input, s.
    utils._BYREF_COMPLEX128, # Input, t.
    utils._EngineFloat64ArrayType.c_type, # Output, zeror[3].
    utils._EngineFloat64ArrayType.c_type, # Output, zeroi[3].
    utils._EngineFloat64ArrayType.c_type, # Output, errest[3].
    utils._BYREF_INT, # Input/Output, ifail.
)

quartic_complex = utils._EngineState._get_symbol('c02anf')
quartic_complex.restype = None
quartic_complex.argtypes = (
    utils._BYREF_COMPLEX128, # Input, e.
    utils._BYREF_COMPLEX128, # Input, a.
    utils._BYREF_COMPLEX128, # Input, b.
    utils._BYREF_COMPLEX128, # Input, c.
    utils._BYREF_COMPLEX128, # Input, d.
    utils._EngineFloat64ArrayType.c_type, # Output, zeror[4].
    utils._EngineFloat64ArrayType.c_type, # Output, zeroi[4].
    utils._EngineFloat64ArrayType.c_type, # Output, errest[4].
    utils._BYREF_INT, # Input/Output, ifail.
)
